@@ -3,7 +3,7 @@ import mimetypes
 import uuid
 import warnings
 from collections.abc import Mapping
-from http.cookiejar import CookieJar
+from http.cookiejar import CookieJar, Cookie
 from urllib.parse import quote_plus, urlencode as _urlencode
 
 from twisted.internet.interfaces import IProtocol
@@ -30,7 +30,7 @@ from json import dumps as json_dumps
 from treq.auth import add_auth
 from treq import multipart
 from treq.response import _Response
-from requests.cookies import cookiejar_from_dict, merge_cookies
+from requests.cookies import merge_cookies
 
 
 _NOTHING = object()
@@ -43,6 +43,56 @@ def urlencode(query, doseq):
     return s
 
 
+def _scoped_cookiejar_from_dict(url_object, cookie_dict):
+    """
+    Create a CookieJar from a dictionary whose cookies are all scoped to the
+    given URL's origin.
+
+    @note: This does not scope the cookies to any particular path, only the
+        host, port, and scheme of the given URL.
+    """
+    cookie_jar = CookieJar()
+    if cookie_dict is None:
+        return cookie_jar
+    for k, v in cookie_dict.items():
+        secure = url_object.scheme == 'https'
+        port_specified = not (
+            (url_object.scheme == "https" and url_object.port == 443)
+            or (url_object.scheme == "http" and url_object.port == 80)
+        )
+        port = str(url_object.port) if port_specified else None
+        domain = url_object.host
+        netscape_domain = domain if '.' in domain else domain + '.local'
+
+        cookie_jar.set_cookie(
+            Cookie(
+                # Scoping
+                domain=netscape_domain,
+                port=port,
+                secure=secure,
+                port_specified=port_specified,
+
+                # Contents
+                name=k,
+                value=v,
+
+                # Constant/always-the-same stuff
+                version=0,
+                path="/",
+                expires=None,
+                discard=False,
+                comment=None,
+                comment_url=None,
+                rfc2109=False,
+                path_specified=False,
+                domain_specified=False,
+                domain_initial_dot=False,
+                rest=[],
+            )
+        )
+    return cookie_jar
+
+
 class _BodyBufferingProtocol(proxyForInterface(IProtocol)):
     def __init__(self, original, buffer, finished):
         self.original = original
@@ -98,7 +148,9 @@ class HTTPClient:
     def __init__(self, agent, cookiejar=None,
                  data_to_body_producer=IBodyProducer):
         self._agent = agent
-        self._cookiejar = cookiejar or cookiejar_from_dict({})
+        if cookiejar is None:
+            cookiejar = CookieJar()
+        self._cookiejar = cookiejar
         self._data_to_body_producer = data_to_body_producer
 
     def get(self, url, **kwargs):
@@ -195,7 +247,7 @@ class HTTPClient:
             headers.setRawHeaders(b'Content-Type', [contentType])
 
         if not isinstance(cookies, CookieJar):
-            cookies = cookiejar_from_dict(cookies)
+            cookies = _scoped_cookiejar_from_dict(parsed_url, cookies)
 
         cookies = merge_cookies(self._cookiejar, cookies)
         wrapped_agent = CookieAgent(self._agent, cookies)
@@ -364,17 +416,18 @@ def _convert_params(params):
 
 
 def _convert_files(files):
-    """Files can be passed in a variety of formats:
+    """
+    Files can be passed in a variety of formats:
 
-        * {'file': open("bla.f")}
-        * {'file': (name, open("bla.f"))}
-        * {'file': (name, content-type, open("bla.f"))}
-        * Anything that has iteritems method, e.g. MultiDict:
-          MultiDict([(name, open()), (name, open())]
+    * {"fieldname": open("bla.f", "rb")}
+    * {"fieldname": ("filename", open("bla.f", "rb"))}
+    * {"fieldname": ("filename", "content-type", open("bla.f", "rb"))}
+    * Anything that has iteritems method, e.g. MultiDict:
+      MultiDict([(name, open()), (name, open())]
 
-        Our goal is to standardize it to unified form of:
+    Our goal is to standardize it to unified form of:
 
-        * [(param, (file name, content type, producer))]
+    * [(param, (file name, content type, producer))]
     """
 
     if hasattr(files, "iteritems"):
@@ -429,11 +482,6 @@ def _query_quote(v):
     if not isinstance(v, bytes):
         v = v.encode("utf-8")
     q = quote_plus(v)
-    if isinstance(q, bytes):
-        # Python 2.7 returnes bytes when given bytes, but Python 3.x always
-        # returns str.  Coverage disabled here to stop Coveralls complaining
-        # until we can drop Python 2.7 support.
-        q = q.decode("ascii")  # pragma: no cover
     return q
 
 
